#ifndef CONFIGFILE_H
#define CONFIGFILE_H
/***************************************************************************
 *
 * ConfigFile.h: a simple configfile
 *
 * (c) 2004 Thomas Unterthiner
 *
 * created:      23. May 2004
 * last change:  $Date: 2004/05/26 16:03:22 $
 *
 ***************************************************************************/

#include <iostream>
#include <fstream>
using std::ifstream;
using std::ofstream;


#include <string>
using std::string;

#include <map>
using std::map;

#include <sstream>

/*
 * A configuration file class
 * stores settings in a file in simple
 * 		KEY ASSIGN_SIGN VALUE
 * lines. Lines beginning with
 *		COMMENT_SIGN
 * will be ignored and can therefore be used as comment-lines
 *
 */
class ConfigFile
{
	public:
		ConfigFile(const string& path);
		~ConfigFile() {};
		void save(const string& newFilename="");
		
		template<class T> bool get(const string& key, T* returnValue);
		
		void set(const string& key, const string& value);
		
	private:

		void readSettings();

		string filepath_;
		map<string, string> settings_;
		static const char COMMENT_SIGN = '#';
		static const char ASSIGN_SIGN = '=';
};



/*
 * search for a setting with a given key.
 * if the key could be found, store it's value in returnValue and return true
 * otherwhise return false.
 * The value will be casted to whatever is needed :)
 */
template<class T> bool ConfigFile::get(const string& key, T* returnValue)
{
	map<string, string>::const_iterator it = settings_.find(key);
	if (it!=settings_.end())
	{
		std::istringstream tmp(it->second);
		tmp >> *returnValue;
		return true;
	}
	return false;
}

#endif

