/***************************************************************************
 *
 * SdlScreensaver.cpp: a screensaver implemented with SDL
 *
 * (c) 2004 Thomas Unterthiner
 *
 * created:      22. May 2004
 * last change:  $Date: 2004/05/26 16:03:22 $
 *
 ***************************************************************************/

#include "SdlScreensaver.h"
#include "SdlSurface.h"

//#define DEBUG

/*
 * create new screensaver with default-settings.
 * TO DO: read settings out of some kind of config-file
 */
SdlScreensaver::SdlScreensaver()
	: run_(true),
	  config_("blueScreen_Saver.conf"),
	  #ifdef DEBUG
	  fullscreen(false)
	  #else
	  fullscreen(true)
	  #endif
{
	// read configuration
	if (!config_.get("xRes", const_cast<int*>(&xRes))) throw badConfigFile();
	if (!config_.get("yRes", const_cast<int*>(&yRes))) throw badConfigFile();
	if (!config_.get("bpp", const_cast<int*>(&bpp))) throw badConfigFile();
	if (!config_.get("rows", const_cast<int*>(&rows))) throw badConfigFile();
	if (!config_.get("cols", const_cast<int*>(&cols))) throw badConfigFile();
	if (!config_.get("speed", const_cast<int*>(&changesPerFrame))) throw badConfigFile();
	if (!config_.get("yRes", const_cast<int*>(&yRes))) throw badConfigFile();
	
	if (SDL_Init( SDL_INIT_VIDEO ) == -1)
	{
		fprintf(stderr, "Can't init SDL: %s\n", SDL_GetError() );
		exit(1);
	}

	display_ = new SdlSurface(xRes, yRes, bpp, fullscreen);
	table_ = new MorphingTable(rows, cols, *display_);
}


/*
 * Dtor
 */
SdlScreensaver::~SdlScreensaver()
{
	delete table_;
	delete display_;
	SDL_Quit();
}



/*
 * run screensaver
 */
void SdlScreensaver::run()
{
	while (run_)
	{
		checkInput();		
		for (int i = 0; i < changesPerFrame; ++i)
		{
			table_->morph();
		}
		table_->draw();
		display_->update();
		display_->clear();
	}
}



/*
 * check for input - all input ends the screensaver
 * NOTE: unable to test for SDL_MOUSEMOTION, because that somehow allways
 *       kills the ScreenSaver instantanously
 */
void SdlScreensaver::checkInput()
{
	SDL_Event event;

	while (SDL_PollEvent (&event))
	{
		switch (event.type)
		{
			case SDL_KEYDOWN:
			case SDL_MOUSEBUTTONDOWN:
			case SDL_MOUSEBUTTONUP:
			case SDL_QUIT:
				run_ = false;
				break;
		}
	}
}

