/***************************************************************************
 *
 * SdlSurface.cpp: drawing-surface implemented with libSDL
 *
 * (c) 2004 Thomas Unterthiner
 *
 * created:      22. May 2004
 * last change:  $Date: 2004/05/26 16:03:22 $
 *
 ***************************************************************************/

#include "SdlSurface.h"
 
/*
 * create a Surface of given resolution (xRes, yRes) and color-depth (bpp)
 *
 * NOTE: SDL has to be already  initiated, but the Video-Subsystem will be
 *       initiated here!
 */ 
SdlSurface::SdlSurface(int xRes, int yRes, int bpp, bool fullscreen)
{

	int flags = SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_ANYFORMAT;
	if (fullscreen)
		flags = flags | SDL_FULLSCREEN;

	surface_ = SDL_SetVideoMode(xRes, yRes, bpp, flags);
	if (surface_ == 0)
	{
		printf( "Can't set video mode: %s\n", SDL_GetError() );
		exit(1);
	}	
}



/*
 * Dtor
 */
SdlSurface::~SdlSurface()
{
	SDL_FreeSurface(surface_);
}


/*
 * return height of the surface
 */
int SdlSurface::getHeight()
{
	return surface_->h;
}


/*
 * return width of the surface
 */
int SdlSurface::getWidth()
{
	return surface_->w;
}

/*
 * update the screen
 */
void SdlSurface::update()
{
	SDL_Flip(surface_);
}



/*
 * clear surface to a black screen
 */
void SdlSurface::clear()
{
	SDL_FillRect(surface_, 0, 0);
}


/*
 * Set the pixel at (x, y) to the given value
 * NOTE: The surface must be locked before calling this!
 */
void SdlSurface::putPixel(int x, int y, Color& color)
{
	int bpp = surface_->format->BytesPerPixel;
	Uint8* p = (Uint8*)surface_->pixels + y * surface_->pitch + x * bpp;
	Uint32 value = SDL_MapRGB(surface_->format, color.r, color.g, color.b);

	switch(bpp)
	{
		case 1:
			*p = value;
			break;
		case 2:
			*(Uint16*)p = value;
			break;
		case 3:
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
			{
				p[0] = (value >> 16) & 0xff;
				p[1] = (value >> 8) & 0xff;
				p[2] = value & 0xff;
			}
			else
			{
				p[0] = value & 0xff;
				p[1] = (value >> 8) & 0xff;
				p[2] = (value >> 16) & 0xff;
			}
			break;
		case 4:
			*(Uint32*)p = value;
			break;
	}
}


/*
 * fill a rectangle witch 2 diagonal edges lay in (x1/y1) and (x2/y2) 
 */
void SdlSurface::fillRect(int x1, int y1, int x2, int y2, Color& color)
{
	SDL_Rect coordinates;
	coordinates.x = x1;
	coordinates.y = y1;
	coordinates.w = x2 - x1;
	coordinates.h = y2 - y1;
	
	SDL_FillRect(surface_,
 	             &coordinates, 
	             SDL_MapRGB(surface_->format, color.r, color.g, color.b));
}


