/***************************************************************************
 *
 * main.cpp: startup-code for the blueScreen-Saver
 *
 * (c) 2004 Thomas Unterthiner
 *
 * created:      22. May 2004
 * last change:  $Date: 2004/05/31 13:32:51 $
 *
 ***************************************************************************/

#include <windows.h>
#include <fstream>
using std::ofstream;

#include <string>
using std::string;

#include "SdlScreensaver.h"

//#define DEBUG

void changeConfiguration(const string& configfile);
void setDefaultConfiguration(const string& configfile);

int main (int argc, char *argv[])
{
	#ifdef DEBUG
	fprintf(stderr, "ARGUMENTS:\n");
	for (int i = 0; i < argc; ++i)
	{
		fprintf(stderr, "%d: %s\n", i, argv[i]);
	}
	#endif

	if (argc < 2) return 1;  // no argument recieved 
	
  	// build path to the configuration file
	string configfile(argv[0]);
	string::size_type endOfPath = configfile.find_last_of('\\', configfile.length());
	if (endOfPath == string::npos) configfile = "blueScreen_Saver.conf";
	configfile.replace(endOfPath, configfile.length(), "\\blueScreen_Saver.conf");
	#ifdef DEBUG
		fprintf(stderr, "configfile: %s\n", configfile.c_str());
	#endif
	

	// screensavers are called with different arguments:
	// /S for screensaver-mode
	// /c for configuration
	// /p for preview
	const char mode = tolower(argv[1][1]);
	switch(mode)
	{
	    case 's':
    		try
    		{
    			SdlScreensaver screensaver;
    			screensaver.run();
    		}
    		catch(badConfigFile error)
    		{
    			MessageBox (NULL,
    			            "having problems reading the configuration...\n"
                            "resetting all to default values", "Error", 0);
    			setDefaultConfiguration(configfile);
    			MessageBox (NULL,
    			            "Configuration has been reset...\n\n"
                            "please restart the screensaver","blueScreen_Saver", 0);
			}
	  		break;

		case 'c':
			changeConfiguration(configfile);
			break;
	}	
	
	return 0;
}



/*
 * launch the editor to directly edit the configfile
 */
void changeConfiguration(const string& configfile)
{
	ShellExecute(0, "open", configfile.c_str(), NULL, ".", SW_SHOWNORMAL);
}



/*
 * create a new configfile
 */
void setDefaultConfiguration(const string& configfile)
{
	ofstream file(configfile.c_str());
	file << "########################################################\n#\n"
	     << "#	blueScreen_Saver - Configurationfile\n#\n"
 	     << "# feel free to mess around as you like, but don't blame\n"
         << "# me for any mistakes you make! ;-)\n"
         << "# lines beginning with # are comments an will be ignored\n"
         << "########################################################\n\n"
         << "# VIDEO-SETTINGS:\n"
         << "#################\n\n"
	     << "# xRes: horizontal screen resolution\n"
         << "xRes = 1024\n\n"
	     << "# yRes: vertical screen resolution\n"
         << "yRes = 768\n\n"
	     << "# color-depth in bit\n"
         << "bpp = 32\n\n"
         << "\n# SCREENSAVER-SETTINGS:\n"
         << "#######################\n\n"
	     << "# cols: No. of columns in the grafic\n"
         << "cols = 64\n\n"
	     << "# rows: No. of rows in the grafic\n"
         << "rows = 48\n\n"
 	     << "# speed: any value higher than cols x rows is useless\n"
         << "speed = 100\n" << std::endl;
	file.close();
}
